<?php

namespace Teknisa\CADEST\Services;


use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;

class ParaVendasImposto {
	
	protected $entityManager;

	public function __construct(\Doctrine\ORM\EntityManager $entityManager)  {
		$this->entityManager = $entityManager;
		$this->connection = $entityManager->getConnection();
	}
	
	public function getTipoImposto($CDFILIAL, $NRORG){
		$params = array( 
			':CDFILIAL'=> $CDFILIAL,
			':NRORG'   => $NRORG
		);
		return $this->entityManager->getConnection()->fetchAll(self::TIPO_IMPOSTO, $params);
	}
	
	public function deleteProdutosImposto($products) {
		try {
			foreach($products as $product) {
				$this->entityManager->getConnection()->executeQuery(self::DELETE_IMPOSTO_PRODUTO, $product);
			}
		} catch(\Exception $e) {
			var_dump($e);
		}
	}
	
	public function deleteProdutoImposto($product) {
		try {
			$params = array(
				"CDPRODUTO" => $product["CDPRODUTO"],
				"NRORG" 	=> $product["NRORG"],
				"CDIMPOSTO" => $product["CDIMPOSTO"],
				"CDFILIAL"	=> $product["CDFILIAL"] 
			);
			$this->entityManager->getConnection()->executeQuery(self::DELETE_IMPOSTO_PRODUTO, $params);
		} catch(\Exception $e) {
			var_dump($e->getMessage());
		}
	}
	
	public function saveProdutosImposto($products, $params) {
		try {	
			foreach($products as $product) {
				$row = array();
				foreach($params as $key => $value) {
					$row[$key] = $value;
				}
				$row['CDPRODUTO'] = $product['CDPRODUTO'];
				$this->entityManager->getConnection()->executeQuery(self::INSERE_IMPOSTO_PRODUTO, $row);
			}
		} catch(\Exception $e) {
			var_dump($e);
		}
	}

	public function saveProdutoImposto($product, $CDFILIAL) {
		try {	
			$row = array();
			foreach($product as $key => $value) {
				$row[$key] = $value;
			}
			$row['CDFILIAL'] = $CDFILIAL;

			$this->entityManager->getConnection()->executeQuery(self::INSERE_IMPOSTO_PRODUTO, $row);
		} catch(\Exception $e) {
			var_dump($e);
		}
	}
	
	public function generateAliquote ($row) {
        $row["P_CDREGRAFISC"]  = $this->buildInputParam("P_CDREGRAFISC", $row["CDFILIAL"]);
        
        $this->genAliProcedure($row);
    }
    
    private function buildInputParam ($paramName, $paramValue) {
        return new Param($paramName, Param::PARAM_INPUT, $paramValue, Param::PARAM_TYPE_STR, strlen($paramValue));
    }
    
    public function genAliProcedure ($params) {
	    try{  
	        $storedProcedure = new StoredProcedure($this->connection, 'GERA_ALIQUOTA_VND');
	        $storedProcedure->addParam($params["P_CDREGRAFISC"]);
	        $storedProcedure->execute();
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function checkImposto($row, $CDFILIAL){
    	$return = array(
    		"showMessage" 		=> false,
			"message" 	=> ''
			);
    	$params = array(
				"NRORG" 		=> $row["NRORG"],
				"CDNVPRODUTO" 	=> $row["CDNVPRODUTO"],
				"CDARVPROD" 	=> $row["CDARVPROD"],
				"CDFILIAL"		=> $CDFILIAL 
			);
        $adqModProdFili = $this->entityManager->getConnection()->fetchAssoc(self::adqModProdFili, $params);
    	$adqGrupoPDV = $this->entityManager->getConnection()->fetchAssoc(self::adqGrupoPDV, $params);
    	$cdSegMer = $adqGrupoPDV['CDSEGMER'];
    	$params['CDSEGMER'] = $adqGrupoPDV['CDSEGMER'];
    	$dataSourceName = '';
    	if($adqModProdFili['IDUTMODPRODFIL'] == 'S'){
    		$filterModProd = $this->entityManager->getConnection()->fetchAssoc(self::filterModProd, $params);
    		if(!$filterModProd){
	    				 $return['showMessage'] = true;
	    				 $return['message'] = "O produto não se encontra em nenhum modulo de produto associado a filial.";
    				}
    	}else{
    		if(!empty($adqGrupoPDV['CDGRPRODPDV'])){
    			if(!empty($adqGrupoPDV['CDSEGMER'])){
    				$params['CDSEGMER'] = $adqGrupoPDV['CDSEGMER'];
    				// Utiliza família / segmento
    				$filterImpProdSegFam = $this->entityManager->getConnection()->fetchAssoc(self::filterImpProdSegFam, $params);
    				if(!$filterImpProdSegFam){
	    				 $return['showMessage'] = true;
	    				 $return['message'] = "O produto não se encontra em nenhuma familia e segmento de produto associado a filial.";
    				}
    			}else{
    				// Utiliza família
    				$filterImpProdFamilia = $this->entityManager->getConnection()->fetchAssoc(self::filterImpProdFamilia, $params);
    				if(!$filterImpProdFamilia){
	    				 $return['showMessage'] = true;
	    				 $return['message'] = "O produto não se encontra em nenhuma familia de produto associado a filial.";
    				}
    			}
    		}else{
    			if(!empty($adqGrupoPDV['CDSEGMER'])){
    				$params['CDSEGMER'] = $adqGrupoPDV['CDSEGMER'];
    				// Utiliza segmento
    				$filterImpProdSegmento = $this->entityManager->getConnection()->fetchAssoc(self::filterImpProdSegmento, $params);
    				if(!$filterImpProdSegmento){
	    				 $return['showMessage'] = true;
	    				 $return['message'] = "O produto não se encontra em nenhum segmento de produto associado a filial.";
    				}
    			}
    		}
    	}
    	return $return;
    }
    
    const TIPO_IMPOSTO= "
        SELECT P.CDSITUTRIB, P.DSSITUTRIB 
          FROM FILIAL F 
          JOIN PARAMEMPRESA PF 
            ON F.CDEMPRESA = PF.CDEMPRESA 
           AND F.NRORG = PF.NRORG 
          JOIN PARAMCST P 
            ON P.NRORG = F.NRORG 
         WHERE (P.CDIMPOSTO = '00') 
           AND (P.IDENTRSAIDOP = 'S') 
           AND PF.CDSITUCRT = '3'
           AND F.CDFILIAL = :CDFILIAL
           AND F.NRORG = :NRORG
    ";
    const DELETE_IMPOSTO_PRODUTO = "
    	DELETE
    		FROM ALIQIMPFIS
    	WHERE CDFILIAL = :CDFILIAL
    		AND NRORG      = :NRORG
    		AND CDIMPOSTO  = :CDIMPOSTO
    		AND CDPRODUTO  = :CDPRODUTO
    ";
    const INSERE_IMPOSTO_PRODUTO = "    
    INSERT INTO ALIQIMPFIS 
      (VRPEALIMPFIS, VRPERCREDUCAO, IDMODALBASECALC, IDINCIDEPISCOF, VRALIQIBPT, 
       VRALIQIBPTES, CDCFOPPFIS, CDCSTICMS, CDCSTPISCOF, VRALIQPIS, VRALIQCOFINS,
       CDCEST, CDIMPOSTO, CDFILIAL, NRORG, CDPRODUTO, IDTPIMPOSFIS)
    VALUES 
      (:VRPEALIMPFIS, :VRPERCREDUCAO, :IDMODALBASECALC, :IDINCIDEPISCOF, :VRALIQIBPT, 
       :VRALIQIBPTES, :CDCFOPPFIS, :CDCSTICMS, :CDCSTPISCOF, :VRALIQPIS, :VRALIQCOFINS,
       :CDCEST, :CDIMPOSTO, :CDFILIAL, :NRORG, :CDPRODUTO, :IDTPIMPOSFIS)
    ";  
    
    const adqGrupoPDV = "
    	SELECT PV.CDGRPRODPDV,
		  FI.CDSEGMER
		FROM FILIAL FI 
		LEFT JOIN PARAVEND PV
		ON (FI.CDFILIAL = PV.CDFILIAL)
		WHERE (FI.CDFILIAL   = :CDFILIAL) AND (FI.NRORG   = :NRORG)
    ";
    
    const adqModProdFili = "
    	SELECT P.CDFILIAL,
		  P.IDUTMODPRODFIL
		FROM PARAMFILIAL P
		WHERE (P.CDFILIAL = :CDFILIAL)
		AND (P.NRORG      = :NRORG)
    ";
    
    const filterModProd = "
    SELECT PS.CDPRODUTO,
       PR.NMPRODUTO,
       PR.CDARVPROD,
       PR.CDBARPRODUTO,
       PR.NRORG,
       A.CDIMPOSTO,
       A.VRPEALIMPFIS,
       A.IDTPIMPOSFIS,
       A.SGIMPOSTO,
       A.NMIMPOSTO,
       A.CDFILIAL,
       A.IDINCIDEPISCOF,
       :CDSEGMER AS CDSEGMER,
       A.VRALIQIBPT,
       A.VRALIQIBPTES,
       A.CDCSTICMS,
       A.CDCSTPISCOF,
       A.CDCFOPPFIS,
       DECODE(A.CDCFOPPFIS, NULL, NULL, A.VRALIQPIS) AS VRALIQPIS,
       DECODE(A.CDCFOPPFIS, NULL, NULL, A.VRALIQCOFINS) AS VRALIQCOFINS,
       A.VRPERCREDUCAO,	
       A.IDMODALBASECALC,
       A.CDCEST
		FROM PRODUTOSEG PG
		JOIN PRODUTO PR ON PR.NRORG = PG.NRORG
		JOIN
		  (SELECT DISTINCT P.CDPRODUTO
		   FROM PRSUBGRPROD P,
		        PARAVEND PE
		   WHERE (PE.CDFILIAL = :CDFILIAL)
		     AND (PE.NRORG = :NRORG)
		     AND (P.CDGRUPPROD = PE.CDGRPRODPDV)
		     AND ((P.CDFILISUBGR = 'P')
		          OR (P.CDFILISUBGR = :CDFILIAL)) ) PS ON (PS.CDPRODUTO = PG.CDPRODUTO)
		AND (PR.CDPRODUTO = PS.CDPRODUTO)
		LEFT JOIN
		  (SELECT COUNT(*) AS NRCOMPPROD,
		          CDPRODUTO
		   FROM ROTFABRICA
		   GROUP BY CDPRODUTO) R ON (R.CDPRODUTO = PR.CDPRODUTO)
		LEFT JOIN
		  (SELECT COUNT(*) AS NRCOMPPRODL,
		          CDPRODUTO
		   FROM ROTFABLOCAL
		   WHERE (CDFILIAL = :CDFILIAL)
		     AND NRORG = :NRORG
		   GROUP BY CDPRODUTO) F ON (F.CDPRODUTO = PR.CDPRODUTO)
		LEFT JOIN
		  (SELECT AL.CDFILIAL,
		          AL.CDPRODUTO,
		          AL.CDIMPOSTO,
		          AL.VRPEALIMPFIS,
		          AL.IDTPIMPOSFIS,
		          IM.SGIMPOSTO,
		          IM.NMIMPOSTO,
		          AL.IDINCIDEPISCOF,
		          AL.VRALIQIBPT,
		          AL.VRALIQIBPTES,
		          AL.CDCSTICMS,
		          AL.CDCSTPISCOF,
		          AL.CDCFOPPFIS,
		          AL.VRALIQPIS,
		          AL.VRALIQCOFINS,
		          AL.VRPERCREDUCAO,
		          AL.IDMODALBASECALC,
		          AL.CDCEST
		   FROM ALIQIMPFIS AL,
		        IMPOSTO IM
		   WHERE AL.CDIMPOSTO = IM.CDIMPOSTO
		     AND AL.CDFILIAL = :CDFILIAL
		     AND AL.NRORG = :NRORG ) A ON (A.CDPRODUTO = PR.CDPRODUTO)
		WHERE (PG.CDSEGMER = :CDSEGMER)
		  AND (PG.NRORG = :NRORG)
		  AND ((PR.CDARVPROD LIKE :CDARVPROD ||'%'))
		  AND (PR.CDNVPRODUTO = :CDNVPRODUTO)
		ORDER BY PR.CDPRODUTO";
		
	const filterImpProdSegFam = "
		SELECT PS.CDPRODUTO,
	       PR.NMPRODUTO,
	       PR.CDARVPROD,
	       PR.CDBARPRODUTO,
	       PR.NRORG,
	       A.CDIMPOSTO,
	       A.VRPEALIMPFIS,
	       A.IDTPIMPOSFIS,
	       A.SGIMPOSTO,
	       A.NMIMPOSTO,
	       A.CDFILIAL,
	       A.IDINCIDEPISCOF,
	       :CDSEGMER AS CDSEGMER,
	       A.VRALIQIBPT,
	       A.VRALIQIBPTES,
	       A.CDCSTICMS,
	       A.CDCSTPISCOF,
	       A.CDCFOPPFIS,
	       DECODE(A.CDCFOPPFIS, NULL, NULL, A.VRALIQPIS) AS VRALIQPIS,
	       DECODE(A.CDCFOPPFIS, NULL, NULL, A.VRALIQCOFINS) AS VRALIQCOFINS,
	       A.VRPERCREDUCAO,
	       A.IDMODALBASECALC,
	       A.CDCEST
	FROM PRODUTOSEG PG JOIN
	     PRODUTO PR ON 
	     PR.NRORG = PG.NRORG
	     JOIN
	  ( SELECT DISTINCT P.CDPRODUTO
	   FROM PRSUBGRPROD P,
	        PARAVEND PE
	   WHERE ( PE.CDFILIAL = :CDFILIAL )
	     AND ( P.NRORG = :NRORG )
	     AND ( P.CDGRUPPROD = PE.CDGRPRODPDV )
	     AND ( ( P.CDFILISUBGR = 'P' )
	          OR ( P.CDFILISUBGR = :CDFILIAL ) ) ) PS ON
	    PS.CDPRODUTO = PR.CDPRODUTO AND
	    PS.CDPRODUTO = PG.CDPRODUTO 
	    
	    LEFT JOIN 
	  ( SELECT COUNT(*) AS NRCOMPPROD,
	           CDPRODUTO
	   FROM ROTFABRICA
	   GROUP BY CDPRODUTO ) R ON
	   ( R.CDPRODUTO = PR.CDPRODUTO )
	  LEFT JOIN
	  ( SELECT COUNT(*) AS NRCOMPPRODL,
	           CDPRODUTO
	   FROM ROTFABLOCAL
	   WHERE ( CDFILIAL = :CDFILIAL )
	   GROUP BY CDPRODUTO ) F ON
	    ( F.CDPRODUTO = PR.CDPRODUTO )
	  LEFT JOIN
	  ( SELECT AL.CDFILIAL,
	           AL.CDPRODUTO,
	           AL.CDIMPOSTO,
	           AL.VRPEALIMPFIS,
	           AL.IDTPIMPOSFIS,
	           IM.SGIMPOSTO,
	           IM.NMIMPOSTO,
	           AL.IDINCIDEPISCOF,
	           AL.VRALIQIBPT,
	           AL.VRALIQIBPTES,
	           AL.CDCSTICMS,
	           AL.CDCSTPISCOF,
	           AL.CDCFOPPFIS,
	           AL.VRALIQPIS,
	           AL.VRALIQCOFINS,
	           AL.VRPERCREDUCAO,
	           AL.IDMODALBASECALC,
	           AL.CDCEST
	   FROM ALIQIMPFIS AL,
	        IMPOSTO IM
	   WHERE AL.CDIMPOSTO = IM.CDIMPOSTO
	     AND AL.CDFILIAL = :CDFILIAL
	     AND AL.NRORG = :NRORG ) A ON
	     ( A.CDPRODUTO = PR.CDPRODUTO )
	WHERE ( PG.CDSEGMER = :CDSEGMER )
	  AND ( PG.NRORG = :NRORG )
	  AND ( PR.CDARVPROD = :CDARVPROD)
	  AND ( PR.CDNVPRODUTO = :CDNVPRODUTO )
	ORDER BY PR.CDPRODUTO";
	
	const filterImpProdFamilia = "
			SELECT PS.CDPRODUTO,
			       PR.CDARVPROD,
			       PR.NMPRODUTO,
			       PR.CDBARPRODUTO,
			       PR.NRORG,
			       A.CDIMPOSTO,
			       A.VRPEALIMPFIS,
			       A.IDTPIMPOSFIS,
			       A.SGIMPOSTO,
			       A.NMIMPOSTO,
			       A.CDFILIAL,
			       A.IDINCIDEPISCOF,
			       :CDSEGMER AS CDSEGMER,
			       A.VRALIQIBPT,
			       A.VRALIQIBPTES,
			       A.CDCSTICMS,
			       A.CDCSTPISCOF,
			       A.CDCFOPPFIS,
			       DECODE(A.CDCFOPPFIS, NULL, NULL, A.VRALIQPIS) AS VRALIQPIS,
			       DECODE(A.CDCFOPPFIS, NULL, NULL, A.VRALIQCOFINS) AS VRALIQCOFINS,
			       A.VRPERCREDUCAO,
			       A.IDMODALBASECALC,
		    	   A.CDCEST
		FROM PRODUTO PR JOIN
		     PARAVEND PG ON
		     PR.NRORG = PG.NRORG
		     JOIN 
		  (SELECT DISTINCT CDPRODUTO,
		                   CDGRUPPROD
		   FROM PRSUBGRPROD
		   WHERE ((CDFILISUBGR = 'P')
		          OR (CDFILISUBGR = :CDFILIAL)) ) PS ON
		     (PR.CDPRODUTO = PS.CDPRODUTO)
		     AND (PG.CDGRPRODPDV = PS.CDGRUPPROD)
		     LEFT JOIN
		  (SELECT COUNT(*) AS NRCOMPPROD,
		          CDPRODUTO
		   FROM ROTFABRICA
		   GROUP BY CDPRODUTO) R ON
		   (PR.CDPRODUTO = R.CDPRODUTO)
		   LEFT JOIN
		  (SELECT COUNT(*) AS NRCOMPPRODL,
		          CDPRODUTO
		   FROM ROTFABLOCAL
		   WHERE (CDFILIAL = :CDFILIAL)
		     AND (NRORG = :NRORG)
		   GROUP BY CDPRODUTO) F ON
		   (PR.CDPRODUTO = F.CDPRODUTO)
		   LEFT JOIN
		  (SELECT AL.CDFILIAL,
		          AL.CDPRODUTO,
		          AL.CDIMPOSTO,
		          AL.VRPEALIMPFIS,
		          AL.IDTPIMPOSFIS,
		          IM.SGIMPOSTO,
		          IM.NMIMPOSTO,
		          AL.IDINCIDEPISCOF,
		          AL.VRALIQIBPT,
		          AL.VRALIQIBPTES,
		          AL.CDCSTICMS,
		          AL.CDCSTPISCOF,
		          AL.CDCFOPPFIS,
		          AL.VRALIQPIS,
		          AL.VRALIQCOFINS,
		          AL.VRPERCREDUCAO,
		          AL.IDMODALBASECALC,
		          AL.CDCEST
		   FROM ALIQIMPFIS AL,
		        IMPOSTO IM
		   WHERE AL.CDIMPOSTO = IM.CDIMPOSTO
		     AND AL.CDFILIAL = :CDFILIAL
		     AND AL.NRORG = :NRORG ) A ON 
		     (PR.CDPRODUTO = A.CDPRODUTO)     
		WHERE (NVL(PR.IDPRODATIVPR, 'S') = 'S')
		  AND (PG.NRORG = :NRORG)
		  AND ((PR.CDARVPROD = :CDARVPROD))
		  AND (PR.CDNVPRODUTO = :CDNVPRODUTO)
		  AND (PG.CDFILIAL = :CDFILIAL)
		ORDER BY PR.CDPRODUTO";
	
	const filterImpProdSegmento = "
		SELECT PR.CDPRODUTO,
	       PR.CDARVPROD,
	       PR.NMPRODUTO,
	       PR.CDBARPRODUTO,
	       PR.NRORG,
	       A.CDIMPOSTO,
	       A.VRPEALIMPFIS,
	       A.IDTPIMPOSFIS,
	       A.SGIMPOSTO,
	       A.NMIMPOSTO,
	       A.CDFILIAL,
	       A.IDINCIDEPISCOF,
	       :CDSEGMER AS CDSEGMER,
	       A.VRALIQIBPT,
	       A.VRALIQIBPTES,
	       A.CDCSTICMS,
	       A.CDCSTPISCOF,
	       A.CDCFOPPFIS,
	       DECODE(A.CDCFOPPFIS, NULL, NULL, A.VRALIQPIS) AS VRALIQPIS,
	       DECODE(A.CDCFOPPFIS, NULL, NULL, A.VRALIQCOFINS) AS VRALIQCOFINS,
	       A.VRPERCREDUCAO,
	       A.IDMODALBASECALC,
	       A.CDCEST
	FROM PRODUTOSEG S JOIN
	     PRODUTO PR ON
	     (PR.CDPRODUTO = S.CDPRODUTO) AND
	     (PR.NRORG     = S.NRORG)
	     LEFT JOIN
	  (SELECT COUNT(*) AS NRCOMPPROD,
	          CDPRODUTO
	   FROM ROTFABRICA
	   GROUP BY CDPRODUTO) R ON
	     (PR.CDPRODUTO = R.CDPRODUTO)
	    LEFT JOIN 
	  (SELECT COUNT(*) AS NRCOMPPRODL,
	          CDPRODUTO
	   FROM ROTFABLOCAL
	   WHERE (CDFILIAL = :CDFILIAL)
	     AND (NRORG = :NRORG)
	   GROUP BY CDPRODUTO) F ON
	     (PR.CDPRODUTO = F.CDPRODUTO)
	    LEFT JOIN
	    (SELECT AL.CDFILIAL,
	          AL.CDPRODUTO,
	          AL.CDIMPOSTO,
	          AL.VRPEALIMPFIS,
	          AL.IDTPIMPOSFIS,
	          IM.SGIMPOSTO,
	          IM.NMIMPOSTO,
	          AL.IDINCIDEPISCOF,
	          AL.VRALIQIBPT,
	          AL.VRALIQIBPTES,
	          AL.CDCSTICMS,
	          AL.CDCSTPISCOF,
	          AL.CDCFOPPFIS,
	          AL.VRALIQPIS,
	          AL.VRALIQCOFINS,
	          AL.VRPERCREDUCAO,
	          AL.IDMODALBASECALC,
	          AL.CDCEST
	   FROM ALIQIMPFIS AL,
	        IMPOSTO IM
	   WHERE AL.CDIMPOSTO = IM.CDIMPOSTO
	     AND AL.CDFILIAL = :CDFILIAL
	     AND AL.NRORG = :NRORG ) A ON
	     (PR.CDPRODUTO = A.CDPRODUTO)     
	WHERE (S.CDSEGMER = :CDSEGMER)
	  AND (S.NRORG = :NRORG)
	  AND (NVL(PR.IDPRODATIVPR, 'S') = 'S')
	  AND (PR.CDARVPROD = :CDARVPROD)
	  AND (PR.CDNVPRODUTO = :CDNVPRODUTO)
	ORDER BY PR.CDPRODUTO
	";
}